/*
 * tmos - hello.c
 * 	version 0.0.0
 * 
 * Copyright (C) 2009  Krzysztof Mazur <krzysiek@podlesie.net>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include <tmos/kernel.h>
#include <tmos/port.h>
#include <tmos/serial.h>
	
static port_t console;

static void putchar(char c) __reentrant
{
	port_write(console, &c, 1);
}

static void puts(const char *s) __reentrant
{
	while (*s)
		putchar(*s++);
}

static const char __code test_str1[] = "0123456789ABCDEFGHIJKLMNOPRQSTUWXYZ\n";

static void test1(void) __reentrant
{
	unsigned char i;
	jiffies_t t;
	unsigned int bps;

	puts("Test 0 - puts performance\n");
	t = get_jiffies();
	for (i = 0; i < 200; i++)
		puts(test_str1);
	t = get_jiffies() - t;
	bps = 200UL * 11 * (sizeof(test_str1) - 1) / jiffies_ms(t);
	printk("TIME test1: %d ms, %d kbps\n", (unsigned int) jiffies_ms(t),
			bps);
	
}

static void test2(void) __reentrant
{
	unsigned char i;
	jiffies_t t;
	unsigned int bps;
	char buf[sizeof(test_str1) - 1];

	puts("Test 2 - write performance\n");
	memcpy(buf, test_str1, sizeof(buf));
	t = get_jiffies();
	for (i = 0; i < 200; i++)
		port_write(console, buf, sizeof(buf));
	t = get_jiffies() - t;
	bps = 200UL * 11 * sizeof(buf) / jiffies_ms(t);
	printk("TIME test2: %d ms, %d kbps\n", (unsigned int) jiffies_ms(t),
			bps);
	
}

static void test3(void) __reentrant
{
	unsigned char i;
	jiffies_t t;
	unsigned int bytes = 0;
	char buf[8];
	msg_ssize_t to_write;
	msg_ssize_t written;
	msg_ssize_t ret;

	puts("Test 2 - write performance\n");
	memcpy(buf, test_str1, sizeof(buf));
	t = get_jiffies();
	do {
		to_write = port_read(console, buf, sizeof(buf));
		if (to_write < 0) {
			puts("error: read error\n");
			return;
		}
		if (!to_write) {
			puts("error: EOF\n");
			return;
		}
		written = 0;
		do {
			ret = port_write(console, buf, to_write);
			if (ret < 0) {
				puts("error: write error\n");
				return;
			}
			if (!ret) {
				puts("error: write returned 0\n");
				return;
			}
			written += ret;
		} while (written < to_write);
		bytes += to_write;
	} while (bytes < 2048);
	t = get_jiffies() - t;
	bytes = 11UL * bytes / jiffies_ms(t);
	printk("TIME test3: %d ms, %d kbps\n", (unsigned int) jiffies_ms(t),
			bytes);
}

int app_main(void) __reentrant
{
	console = port_open(CONFIG_CONSOLE_DEVICE, O_RDWR);
	if (console == -1)
		BUG();

	puts("Console test\n");
	
	test1();
	test2();
	test3();
	for (;;)
		schedule();
}
