/*
 * wysmod - jiffies.c
 * 	version 0.0.0
 * 
 * Copyright (C) 2005, 2006, 2007  Krzysztof Mazur <krzysiek@podlesie.net>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include <tmos/kernel.h>
#include <tmos/config.h>
#include <tmos/jiffies.h>
#include <tmos/serial.h>
#include <asm/system.h>
#include <asm/8051.h>
#include <asm/softirq.h>
#include <tmos/sched.h>

#define __RELOAD_VALUE (CONFIG_OSCILLATOR / CONFIG_TIMER_DIVISOR / HZ)
#define RELOAD_VALUE ((uint16_t)(65536 - (__RELOAD_VALUE)))

struct __check_reload_value {
	char a[(__RELOAD_VALUE < 65536) ? 1 : -1];
};

jiffies_t jiffies = (jiffies_t)(-30 * HZ);
#if CONFIG_JIFFIES_SEQLOCK
uint8_t jiff_seq = 0;

/*
 * FIXME: before jiff_seq++ should be smp_wmb(), but current barrier()
 * implementation causes storing too many registers.
 *
 * Current versions of SDCC compiles good code without smp_wmb().
 * jiff_seq must be incremented AFTER incrementing jiffies. This
 * important not only on SMP (currently not supported), but also
 * when timer interrupt is normal interrupt and high priority
 * interrupt occur.
 */
#define jiff_seq_inc()	do { jiff_seq++; } while (0)

#else
#define jiff_seq_inc()
#endif

extern struct context __kernel *current;

void timer_irq(void) __interrupt(1) __using(TIMER_BANK) __naked
{
	irq_enter(TIMER_BANK);
	TL0 = RELOAD_VALUE & 0xff;
	TH0 = RELOAD_VALUE >> 8;

	jiffies++;
	jiff_seq_inc();

#ifdef CONFIG_TASK_TIMES
	if (task_is_running(current))
		current->cpu_time++;
#endif

	/* FIXME: add generic timers */
#ifdef CONFIG_TRT_BIMP_VELOCITY
	/*
	 * to call any functions we need to save also DPL and DPH.
	 * ACC, B, _BP and PSW are already saved. The R0-R7 use other
	 * register bank and don't need to be saved.
	 */
	__asm__(push dph);
	__asm__(push dpl);
	trt_bimp_irq();
	__asm__(pop dpl);
	__asm__(pop dph);
#endif

	set_timers_pending();
	set_need_resched();
	irq_exit(TIMER_BANK);
}

#if CONFIG_BITS_PER_JIFFIES != 8
jiffies_t get_jiffies(void) REENTRANT
{
	jiffies_t ret;

	preempt_disable();
	ET0 = 0;
	ret = jiffies;
	ET0 = 1;
	preempt_enable();
	return ret;
}
#endif

void jiffies_init(void)
{
	TR0 = 0;
	TMOD = (TMOD & ~0x0f) | 0x01;
	TL0 = RELOAD_VALUE & 0xff;
	TH0 = RELOAD_VALUE >> 8;
	TR0 = 1;
	TF0 = 1;
	ET0 = 1;
}
