/*
 * tmos - at89s52.c
 *
 * AT89S52 watchdog driver
 *
 * Copyright (C) 2013  Krzysztof Mazur <krzysiek@podlesie.net>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 */

#include <tmos/kernel.h>
#include <tmos/watchdog.h>
#include <asm/sfr.h>

DEFINE_SFR(AT89S52_WDTRST, 0xa6)

#define WDT_SEQ1	0x1e
#define WDT_SEQ2	0xe1

void watchdog_refresh(void) __naked
{
#if __INLINE_ASM
	__asm
		mov	c, _EA
		clr	_EA
		mov	_AT89S52_WDTRST, #WDT_SEQ1
		mov	_AT89S52_WDTRST, #WDT_SEQ2
		mov	_EA, c
		ret
	__endasm;
#else
	cpu_flags_t flags;

	local_irq_save(flags);
	AT89S52_WDTRST = WDT_SEQ1;
	AT89S52_WDTRST = WDT_SEQ2;
	local_irq_restore(flags);
#endif
}

void watchdog_init(void)
{
	watchdog_refresh();
}
