/*
 * tmos - jiffies.h
 *
 * Copyright (C) 2005..2011  Krzysztof Mazur <krzysiek@podlesie.net>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#ifndef __ASM_JIFFIES_H
#define __ASM_JIFFIES_H

#include <tmos/kernel.h>
#include <tmos/config.h>
#include <asm/system.h>

#define TIMER_BANK		LOW_PRIORITY_BANK

#if CONFIG_BITS_PER_JIFFIES == 0
typedef uint8_t jiffies_t;
typedef int8_t sjiffies_t;
#elif CONFIG_BITS_PER_JIFFIES == 8
typedef uint8_t jiffies_t;
typedef int8_t sjiffies_t;
#elif CONFIG_BITS_PER_JIFFIES == 16
typedef uint16_t jiffies_t;
typedef int16_t sjiffies_t;
#elif CONFIG_BITS_PER_JIFFIES == 32
typedef uint32_t jiffies_t;
typedef int32_t sjiffies_t;
#else
#error You lose.
#endif

extern jiffies_t jiffies;

/*
 * use only from timer interrupt callbacks or when non-atomics reads
 * are acceptable
 */
#define __get_jiffies()	(jiffies + 0)

#if CONFIG_BITS_PER_JIFFIES == 8
#define get_jiffies()	(jiffies + 0)
#else
jiffies_t get_jiffies(void) __reentrant;
#endif

#define time_after(a, b)	((sjiffies_t)(b) - (sjiffies_t)(a) < 0)
#define time_after_eq(a, b)	((sjiffies_t)(a) - (sjiffies_t)(b) >= 0)

void jiffies_init(void);

void jsleep(jiffies_t t) REENTRANT;

typedef u16 tsc_t;

u16 tsc_read(void);
u16 tsc_read_low(void) __using(LOW_PRIORITY_BANK);

#ifdef CONFIG_TRT_BIMP
void trt_bimp_irq(void) __using(TIMER_BANK) __reentrant;
#endif

#endif
