/* Integer base 2 logarithm calculation
 *
 * Copyright (C) 2006 Red Hat, Inc. All Rights Reserved.
 * Written by David Howells (dhowells@redhat.com)
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version
 * 2 of the License, or (at your option) any later version.
 */

#ifndef _LINUX_LOG2_H
#define _LINUX_LOG2_H

#include <tmos/types.h>

/*
 * deal with unrepresentable constant logarithms
 */
extern __attribute__((const, noreturn))
int ____ilog2_NaN(void);

extern unsigned char __fls_u8(u8 n);
extern unsigned char __fls_u16(u16 n);
extern unsigned char __fls_u32(u32 n);
		
#define __ilog2_u8(n) (__fls_u8((n)) - 1)
#define __ilog2_u16(n) (__fls_u16((n)) - 1)
#define __ilog2_u32(n) (__fls_u32((n)) - 1)

/**
 * ilog2 - log of base 2 of 32-bit or a 64-bit unsigned value
 * @n - parameter
 *
 * constant-capable log of base 2 calculation
 * - this can be used to initialise global variables from constant data, hence
 *   the massive ternary operator construction
 *
 * selects the appropriately-sized optimised version depending on sizeof(n)
 */
#define ilog2(n)				\
(						\
	__builtin_constant_p(n) ? (		\
		(n) < 1 ? ____ilog2_NaN() :	\
		(n) & (1UL << 31) ? 31 :	\
		(n) & (1UL << 30) ? 30 :	\
		(n) & (1UL << 29) ? 29 :	\
		(n) & (1UL << 28) ? 28 :	\
		(n) & (1UL << 27) ? 27 :	\
		(n) & (1UL << 26) ? 26 :	\
		(n) & (1UL << 25) ? 25 :	\
		(n) & (1UL << 24) ? 24 :	\
		(n) & (1UL << 23) ? 23 :	\
		(n) & (1UL << 22) ? 22 :	\
		(n) & (1UL << 21) ? 21 :	\
		(n) & (1UL << 20) ? 20 :	\
		(n) & (1UL << 19) ? 19 :	\
		(n) & (1UL << 18) ? 18 :	\
		(n) & (1UL << 17) ? 17 :	\
		(n) & (1UL << 16) ? 16 :	\
		(n) & (1UL << 15) ? 15 :	\
		(n) & (1UL << 14) ? 14 :	\
		(n) & (1UL << 13) ? 13 :	\
		(n) & (1UL << 12) ? 12 :	\
		(n) & (1UL << 11) ? 11 :	\
		(n) & (1UL << 10) ? 10 :	\
		(n) & (1UL <<  9) ?  9 :	\
		(n) & (1UL <<  8) ?  8 :	\
		(n) & (1UL <<  7) ?  7 :	\
		(n) & (1UL <<  6) ?  6 :	\
		(n) & (1UL <<  5) ?  5 :	\
		(n) & (1UL <<  4) ?  4 :	\
		(n) & (1UL <<  3) ?  3 :	\
		(n) & (1UL <<  2) ?  2 :	\
		(n) & (1UL <<  1) ?  1 :	\
		(n) & (1UL <<  0) ?  0 :	\
		____ilog2_NaN()			\
				   ) :		\
	(sizeof(n) <= 2) ?			\
		((sizeof(n) <= 1) ?		\
		__ilog2_u8(n) :			\
		__ilog2_u16(n)) :		\
	__ilog2_u32(n)				\
 )

#endif /* _LINUX_LOG2_H */
