/*
 * tmos - sched.h
 *
 * Copyright (C) 2009  Krzysztof Mazur <krzysiek@podlesie.net>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#ifndef __SCHED_H
#define __SCHED_H

#include <asm/task.h>
#include <setjmp.h>

#define TASK_COMM_LEN	16

struct task {
	struct context __kernel *context;
	pid_t pid;
	char comm[TASK_COMM_LEN];
};

#if 1
extern const struct task __code __task_array[];

#define DECLARE_TASK_POINTER(name) \
	const struct task __code *name

#define for_each_task(task) \
	for (task = __task_array; task->context != KERNEL_NULL; task++)

#define BEGIN_TASK_ARRAY \
	const struct task __code __task_array[] = {

#define TASK(_pid, _comm, _context) \
		{ \
			_context, \
			_pid, \
			_comm, \
		}

#define END_TASK_ARRAY  \
		TASK(1, "", KERNEL_NULL), \
	};

void sched_init(void);

#else
#endif

char task_init(struct task __kernel *task);

#if CONFIG_WATCHDOG_SCHEDULE
#include <tmos/watchdog.h>

#define watchdog_schedule()	watchdog_refresh()
#else
#define watchdog_schedule()	do { } while (0)
#endif


#ifdef CONFIG_MULTITASKING
void schedule(void);
void wake_up_context(struct context __kernel *context) __reentrant;
void __remove_run_queue(struct context __kernel *context) __reentrant;

void schedule_timeout(unsigned char timeout);

pid_t pid_alloc(void) __reentrant;

void __exit(void);
#define _exit(status)	do { (void) (status); __exit(); } while (0)
#define exit(status) _exit(status)

/**
 * tfork - create a child task
 * 
 * The tfork() is a limited implementation of UNIX fork(2). It is
 * much closer to vfork(2) with exception that instead of exec(2)
 * another subroutine should be called.
 *
 * The tfork() have following limitations:
 *
 * 	- the stack is not fully copied, the child cannot return
 * 	  from routine that called tfork(), additionally accessing
 * 	  local variables from child process is not a good idea.
 * 	  The best is to call immediately another subroutine.
 *
 * 	- all resources, except for stack, CPU context and some
 * 	  part of memory when CONFIG_TDATA is set are shared
 * 	  between both tasks.
 */
static inline pid_t tfork(void) __reentrant
{
	extern char __tfork(void);
	pid_t pid;

	pid = __tfork();
	if (pid < 0)
		return pid;

	if (!thread_create(&task[pid], task_stack(pid))) {
		preempt_enable();
		return pid;
	}
	return 0;
}

/**
 * __wake_up_context - wake up process from interrupt
 *
 * This function must be called with interrupts disabled
 */
#define __wake_up_context(ctx)						\
	do {								\
		struct context __kernel *c = current->next;		\
									\
		/* add new task after last task */			\
		set_task_state((ctx), TASK_RUNNING);			\
		(ctx)->next = current;					\
		c->next = (ctx);					\
	} while (0)
#else
#define sched_init() do { } while (0)
#define schedule() \
	do { \
		watchdog_schedule(); \
		idle(); \
	} while (0)
#endif

#endif
