/*
 * tmos - skb.c
 *
 * spruked protocol for i51 serial
 *
 * Copyright (C) 2013  Krzysztof Mazur <krzysiek@podlesie.net>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 */

#include <tmos/spruked.h>
#include <tmos/ipage.h>
#include <asm/irqs.h>

struct spruked_skb __kernel *skbs_free;

void spruked_consume_skb(struct spruked_skb __kernel *skb)
{
	local_irq_disable();
	__spruked_consume_skb(skb);
	local_irq_enable();
}

struct spruked_skb __kernel *spruked_alloc_skb(void)
{
	struct spruked_skb __kernel *skb;

	local_irq_disable();
	skb = skbs_free;
	if (skb)
		skbs_free = skb->next;
	local_irq_enable();
	return skb;
}

static void spruked_skb_init(void)
{
	u8 __kernel *i;

	i = ipage_alloc();
	do {
		__spruked_kfree_skb((struct spruked_skb __kernel *) i);
		i += 4;
	} while ((unsigned char) i & (IPAGE_SIZE - 1));
}
subsys_initcall(spruked_skb_init)
